Package: irlba
Type: Package
Title: Fast Truncated Singular Value Decomposition and Principal
        Components Analysis for Large Dense and Sparse Matrices
Version: 2.3.7
Date: 2026-1-26
Authors@R: c(
    person("Jim", "Baglama", role=c("aut", "cph"), email="jbaglama@uri.edu"),
    person("Lothar", "Reichel", role=c("aut", "cph"), email="reichel@math.kent.edu"),
    person("B. W.", "Lewis", role=c("aut","cre","cph"), email="blewis@illposed.net"))
Description: Fast and memory efficient methods for truncated singular value
    decomposition and principal components analysis of large sparse and dense
    matrices.
Depends: R (>= 3.6.2), Matrix
LinkingTo: Matrix
Imports: stats, methods
License: GPL-3
BugReports: https://github.com/bwlewis/irlba/issues
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-01-27 21:45:00 UTC; blewis
Author: Jim Baglama [aut, cph],
  Lothar Reichel [aut, cph],
  B. W. Lewis [aut, cre, cph]
Maintainer: B. W. Lewis <blewis@illposed.net>
Repository: CRAN
Encoding: UTF-8
Date/Publication: 2026-01-30 07:20:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 23:18:57 UTC; unix
Archs: irlba.so.dSYM
