# Generated by roxygen2: do not edit by hand

S3method(summary,irlba_prcomp)
export(irlba)
export(partial_eigen)
export(prcomp_irlba)
export(ssvd)
export(svdr)
import(Matrix)
importFrom(methods,slot)
importFrom(methods,slotNames)
importFrom(stats,prcomp)
importFrom(stats,rnorm)
importFrom(stats,sd)
importFrom(stats,var)
useDynLib(irlba, .registration=TRUE, .fixes="C_")
