% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prcomp.R
\name{summary.irlba_prcomp}
\alias{summary.irlba_prcomp}
\title{Summary method for truncated pca objects computed by \code{prcomp_irlba}.}
\usage{
\method{summary}{irlba_prcomp}(object, ...)
}
\arguments{
\item{object}{An object returned by \code{prcomp_irlba}.}

\item{...}{Optional arguments passed to \code{summary}.}
}
\description{
Summary method for truncated pca objects computed by \code{prcomp_irlba}.
}

